@@ -7,7 +7,8 @@ from pages.utils import get_placeholders
 from pages.templatetags.pages_tags import PlaceholderNode
 from pages.admin.utils import get_connected, make_inline_admin
 from pages.admin.forms import PageForm
-from pages.admin.views import traduction, get_content, sub_menu, list_pages_ajax
+from pages.admin.views import traduction, get_content, sub_menu
+from pages.admin.views import list_pages_ajax
 from pages.admin.views import change_status, modify_content, delete_content
 from pages.admin.views import move_page
 from pages.permissions import PagePermission
@@ -42,7 +43,7 @@ class PageAdmin(admin.ModelAdmin):
     if settings.PAGE_USE_SITE_ID and not settings.PAGE_HIDE_SITES:
         general_fields.append('sites')
     insert_point = general_fields.index('status') + 1
-    
+
     # Strange django behavior. If not provided, django will try to find
     # 'page' foreign key in all registered models
     inlines = []
@@ -115,7 +116,7 @@ class PageAdmin(admin.ModelAdmin):
                 change_status, name='page-change-status'),
         )
         urlpatterns += super(PageAdmin, self).urls
-        
+
         return urlpatterns
 
     urls = property(urls)
@@ -162,7 +163,7 @@ class PageAdmin(admin.ModelAdmin):
                 extra_data = placeholder.get_extra_data(form.data)
                 placeholder.save(page, language, data, change,
                     extra_data=extra_data)
-        
+
         page.invalidate()
 
     def get_fieldsets(self, request, obj=None):
@@ -256,6 +257,11 @@ class PageAdmin(admin.ModelAdmin):
             #'lang': current_lang,
             'page_languages': settings.PAGE_LANGUAGES,
         }
+        try:
+            int(object_id)
+        except ValueError:
+            raise Http404('The "%s" part of the location is invalid.'
+                % str(object_id))
         try:
             obj = self.model.objects.get(pk=object_id)
         except self.model.DoesNotExist:
@@ -308,7 +314,7 @@ class PageAdmin(admin.ModelAdmin):
         if not admin.site.has_permission(request):
             return admin.site.login(request)
         language = get_language_from_request(request)
-        
+
         query = request.POST.get('q', '').strip()
 
         if query:
