@@ -9,6 +9,7 @@ from pages import settings
 
 LANGUAGE_KEYS = [key for (key, value) in settings.PAGE_LANGUAGES]
 
+
 def get_request_mock():
     """Build a ``request`` mock up that can be used for testing."""
     basehandler = BaseHandler()
@@ -19,7 +20,6 @@ def get_request_mock():
         'SERVER_PORT': '8000',
         'HTTP_HOST': 'testhost',
     })
-    request.path = reverse('pages-root')
     # Apply request middleware
     for middleware_method in basehandler._request_middleware:
         # LocaleMiddleware should never be applied a second time because
@@ -28,11 +28,13 @@ def get_request_mock():
             response = middleware_method(request)
     return request
 
+
 class AutoRenderHttpError(Exception):
     """Cannot return context dictionary because a view returned an
     ``HttpResponse`` when a (template_name, context) tuple was expected."""
     pass
 
+
 def auto_render(func):
     """
     This view decorator automatically calls the ``render_to_response``
@@ -49,27 +51,34 @@ def auto_render(func):
             if only_response:
                 return response
             if isinstance(response, HttpResponse):
-                raise AutoRenderHttpError
+                raise AutoRenderHttpError(AutoRenderHttpError.__doc__)
             (template_name, context) = response
             return context
         response = func(request, *args, **kwargs)
         if isinstance(response, HttpResponse):
             return response
         (template_name, context) = response
-        template_name = context['template_name'] = template_override or template_name
+        template_name = context['template_name'] = (template_override or
+            template_name)
         return render_to_response(template_name, context,
                             context_instance=RequestContext(request))
     return auto_render_decorator
 
+
 def pages_view(view):
     """
-    Provide the essential pages variables to the decorated view
-    by using the default pages.views.details view.
+    Make sure the decorated view gets the essential pages
+    variables.
     """
     def pages_view_decorator(request, *args, **kwargs):
+        # if the current page is already there
+        if(kwargs.get('current_page', False) or
+            kwargs.get('pages_navigation', False)):
+            return view(request, *args, **kwargs)
+
         path = kwargs.pop('path', None)
         lang = kwargs.pop('lang', None)
-        if path or lang:
+        if path:
             from pages.views import details
             response = details(request, path=path, lang=lang,
                 only_context=True, delegation=False)
@@ -79,27 +88,34 @@ def pages_view(view):
     return pages_view_decorator
 
 
-def get_slug_and_relative_path(path, lang=None):
+def get_slug(path):
     """
-    Return the page's slug, relative path and language.
-    If the language prefix is used in the pathm, this function
-    will try to remove the language from the path.
-    
-        >>> get_slug_and_relative_path('/test/function/')
-        ('function', 'test/function', None)
+    Return the page's slug
+
+        >>> get_slug('/test/function/')
+        function
     """
-    root = reverse('pages-root')
-    if path.startswith(root):
-        path = path[len(root):]
-    if len(path) and path[-1] == '/':
+    if path.endswith('/'):
         path = path[:-1]
-    slug = path.split("/")[-1]
-    if settings.PAGE_USE_LANGUAGE_PREFIX:
-        maybe_lang = path.split("/")[0]
-        if maybe_lang in LANGUAGE_KEYS:
-            lang = maybe_lang
-            path = path[(len(lang) + 1):]
-    return slug, path, lang
+    return path.split("/")[-1]
+
+
+def remove_slug(path):
+    """
+    Return the remainin part of the path
+
+        >>> remove_slug('/test/some/function/')
+        test/some
+    """
+    if path.endswith('/'):
+        path = path[:-1]
+    if path.startswith('/'):
+        path = path[1:]
+    if "/" not in path or not path:
+        return None
+    parts = path.split("/")[:-1]
+    return "/".join(parts)
+
 
 def get_template_from_request(request, page=None):
     """
@@ -118,6 +134,7 @@ def get_template_from_request(request, page=None):
         return page.get_template()
     return settings.PAGE_DEFAULT_TEMPLATE
 
+
 def get_language_from_request(request):
     """Return the most obvious language according the request."""
     language = request.GET.get('language', None)
@@ -132,4 +149,3 @@ def get_language_from_request(request):
             return lang
     else:
         return settings.PAGE_DEFAULT_LANGUAGE
-
