@@ -14,12 +14,14 @@ from django.core.cache import cache
 from django.core.urlresolvers import reverse
 from django.contrib.sites.models import Site
 
-import mptt
+from mptt.models import MPTTModel
+if settings.PAGE_TAGGING:
+    from taggit.managers import TaggableManager
 
 PAGE_CONTENT_DICT_KEY = ContentManager.PAGE_CONTENT_DICT_KEY
 
 
-class Page(models.Model):
+class Page(MPTTModel):
     """
     This model contain the status, dates, author, template.
     The real content of the page can be found in the
@@ -45,7 +47,7 @@ class Page(models.Model):
     .. attribute:: template
        A string containing the name of the template file for this page.
     """
-    
+
     # some class constants to refer to, e.g. Page.DRAFT
     DRAFT = 0
     PUBLISHED = 1
@@ -58,19 +60,19 @@ class Page(models.Model):
     )
 
     PAGE_LANGUAGES_KEY = "page_%d_languages"
-    PAGE_URL_KEY = "page_%d_language_%s_url"
+    PAGE_URL_KEY = "page_%d_url"
     PAGE_BROKEN_LINK_KEY = "page_broken_link_%s"
 
     author = models.ForeignKey(User, verbose_name=_('author'))
-    
-    parent = models.ForeignKey('self', null=True, blank=True, 
+
+    parent = models.ForeignKey('self', null=True, blank=True,
             related_name='children', verbose_name=_('parent'))
     creation_date = models.DateTimeField(_('creation date'), editable=False,
             default=datetime.now)
-    publication_date = models.DateTimeField(_('publication date'), 
+    publication_date = models.DateTimeField(_('publication date'),
             null=True, blank=True, help_text=_('''When the page should go
             live. Status must be "Published" for page to go live.'''))
-    publication_end_date = models.DateTimeField(_('publication end date'), 
+    publication_end_date = models.DateTimeField(_('publication end date'),
             null=True, blank=True, help_text=_('''When to expire the page.
             Leave empty to never expire.'''))
 
@@ -86,9 +88,9 @@ class Page(models.Model):
     freeze_date = models.DateTimeField(_('freeze date'),
             null=True, blank=True, help_text=_('''Don't publish any content
             after this date.'''))
-    
+
     if settings.PAGE_USE_SITE_ID:
-        sites = models.ManyToManyField(Site, default=[settings.SITE_ID], 
+        sites = models.ManyToManyField(Site, default=[settings.SITE_ID],
                 help_text=_('The site(s) the page is accessible at.'),
                 verbose_name=_('sites'))
 
@@ -96,13 +98,12 @@ class Page(models.Model):
 
     redirect_to = models.ForeignKey('self', null=True, blank=True,
             related_name='redirected_pages')
-    
+
     # Managers
     objects = PageManager()
 
     if settings.PAGE_TAGGING:
-        from tagging import fields
-        tags = fields.TagField(null=True)
+        tags = TaggableManager()
 
     class Meta:
         """Make sure the default page ordering is correct."""
@@ -159,6 +160,11 @@ class Page(models.Model):
         return self.status
     calculated_status = property(_get_calculated_status)
 
+    def _visible(self):
+        """Return True if the page is visible on the frontend."""
+        return self.calculated_status in (self.PUBLISHED, self.HIDDEN)
+    visible = property(_visible)
+
     def get_children_for_frontend(self):
         """Return a :class:`QuerySet` of published children page"""
         return Page.objects.filter_published(self.get_children())
@@ -190,10 +196,7 @@ class Page(models.Model):
             cache.delete(PAGE_CONTENT_DICT_KEY %
                 (self.id, name, 0))
 
-        for lang in settings.PAGE_LANGUAGES:
-            cache.delete(self.PAGE_URL_KEY % (self.id, lang[0]))
-        cache.delete(self.PAGE_URL_KEY % (self.id, "None"))
-
+        cache.delete(self.PAGE_URL_KEY % (self.id))
 
     def get_languages(self):
         """
@@ -202,23 +205,27 @@ class Page(models.Model):
         if self._languages:
             return self._languages
         self._languages = cache.get(self.PAGE_LANGUAGES_KEY % (self.id))
-        if self._languages:
+        if self._languages is not None:
             return self._languages
 
         languages = [c['language'] for
                             c in Content.objects.filter(page=self,
                             type="slug").values('language')]
-        languages = list(set(languages)) # remove duplicates
+        # remove duplicates
+        languages = list(set(languages))
         languages.sort()
         cache.set(self.PAGE_LANGUAGES_KEY % (self.id), languages)
         self._languages = languages
         return languages
 
     def is_first_root(self):
-        """Return ``True`` if the page is the first root page."""
+        """Return ``True`` if the page is the first root pages."""
         if self.parent:
             return False
-        return Page.objects.root()[0].id == self.id
+        root_pages = Page.objects.root()
+        if len(root_pages):
+            return Page.objects.root()[0].id == self.id
+        return False
 
     def get_url_path(self, language=None):
         """Return the URL's path component. Add the language prefix if
@@ -226,10 +233,21 @@ class Page(models.Model):
 
         :param language: the wanted url language.
         """
-        url = reverse('pages-root')
+        if self.is_first_root():
+            # this is used to allow users to change URL of the root
+            # page. The language prefix is not usable here.
+            try:
+                return reverse('pages-root')
+            except Exception:
+                pass
+        url = self.get_complete_slug(language)
+        if not language:
+            language = settings.PAGE_DEFAULT_LANGUAGE
         if settings.PAGE_USE_LANGUAGE_PREFIX:
-            url += str(language) + '/'
-        return url + self.get_complete_slug(language)
+            return reverse('pages-details-by-path',
+                args=[language, url])
+        else:
+            return reverse('pages-details-by-path', args=[url])
 
     def get_absolute_url(self, language=None):
         """Alias for `get_url_path`.
@@ -246,21 +264,27 @@ class Page(models.Model):
         all parent's slugs.
 
         :param language: the wanted slug language."""
-        if self._complete_slug:
-            return self._complete_slug
-        self._complete_slug = cache.get(self.PAGE_URL_KEY %
-            (self.id, language))
-        if self._complete_slug:
-            return self._complete_slug
+        if not language:
+            language = settings.PAGE_DEFAULT_LANGUAGE
+
+        if self._complete_slug and language in self._complete_slug:
+            return self._complete_slug[language]
+
+        self._complete_slug = cache.get(self.PAGE_URL_KEY % (self.id))
+        if self._complete_slug is None:
+            self._complete_slug = {}
+        elif language in self._complete_slug:
+            return self._complete_slug[language]
+
         if settings.PAGE_HIDE_ROOT_SLUG and self.is_first_root():
-            url = ''
+            url = u''
         else:
             url = u'%s' % self.slug(language)
         for ancestor in self.get_ancestors(ascending=True):
             url = ancestor.slug(language) + u'/' + url
 
-        cache.set(self.PAGE_URL_KEY % (self.id, language), url)
-        self._complete_slug = url
+        self._complete_slug[language] = url
+        cache.set(self.PAGE_URL_KEY % (self.id), self._complete_slug)
         return url
 
     def get_url(self, language=None):
@@ -281,7 +305,7 @@ class Page(models.Model):
         :param fallback: if ``True``, the slug will also be searched in other \
         languages.
         """
-        
+
         slug = self.get_content(language, 'slug', language_fallback=fallback)
 
         return slug
@@ -296,7 +320,7 @@ class Page(models.Model):
         """
         if not language:
             language = settings.PAGE_DEFAULT_LANGUAGE
-            
+
         return self.get_content(language, 'title', language_fallback=fallback)
 
     def get_content(self, language, ctype, language_fallback=False):
@@ -317,12 +341,31 @@ class Page(models.Model):
         placeholders = get_placeholders(self.get_template())
         exposed_content = []
         for lang in self.get_languages():
-            for p_name in [p.name for p in placeholders]:
-                content = self.get_content(lang, p_name, False)
+            for ctype in [p.name for p in placeholders]:
+                content = self.get_content(lang, ctype, False)
                 if content:
                     exposed_content.append(content)
         return u"\r\n".join(exposed_content)
 
+    def content_by_language(self, language):
+        """
+        Return a list of latest published
+        :class:`Content <pages.models.Content>`
+        for a particluar language.
+
+        :param language: wanted language,
+        """
+        placeholders = get_placeholders(self.get_template())
+        content_list = []
+        for ctype in [p.name for p in placeholders]:
+            try:
+                content = Content.objects.get_content_object(self,
+                    language, ctype)
+                content_list.append(content)
+            except Content.DoesNotExist:
+                pass
+        return content_list
+
     def get_template(self):
         """
         Get the :attr:`template <Page.template>` of this page if
@@ -393,20 +436,14 @@ class Page(models.Model):
             slug = self.slug()
             if slug:
                 return slug
-            return "Page %d" % self.id
-        return "Page without id"
-
-# Don't register the Page model twice.
-try:
-    mptt.register(Page)
-except mptt.AlreadyRegistered:
-    pass
+            return u"Page %d" % self.id
+        return u"Page without id"
 
 
 class Content(models.Model):
     """A block of content, tied to a :class:`Page <pages.models.Page>`,
     for a particular language"""
-    
+
     # languages could have five characters : Brazilian Portuguese is pt-br
     language = models.CharField(_('language'), max_length=5, blank=False)
     body = models.TextField(_('body'))
@@ -424,7 +461,7 @@ class Content(models.Model):
         verbose_name_plural = _('contents')
 
     def __unicode__(self):
-        return "%s :: %s" % (self.page.slug(), self.body[0:15])
+        return u"%s :: %s" % (self.page.slug(), self.body[0:15])
 
 
 class PageAlias(models.Model):
@@ -433,14 +470,14 @@ class PageAlias(models.Model):
         verbose_name=_('page'))
     url = models.CharField(max_length=255, unique=True)
     objects = PageAliasManager()
+
     class Meta:
         verbose_name_plural = _('Aliases')
-    
+
     def save(self, *args, **kwargs):
         # normalize url
         self.url = normalize_url(self.url)
         super(PageAlias, self).save(*args, **kwargs)
-    
-    def __unicode__(self):
-        return "%s => %s" % (self.url, self.page.get_complete_slug())
 
+    def __unicode__(self):
+        return u"%s :: %s" % (self.url, self.page.get_complete_slug())
