@@ -7,6 +7,7 @@ from django.core.exceptions import ImproperlyConfigured
 
 url = 'http://packages.python.org/django-page-cms/settings-list.html#%s'
 
+
 def get_setting(*args, **kwargs):
     """Get a setting and raise an appropriate user friendly error if
     the setting is not found."""
@@ -45,15 +46,16 @@ PAGE_DEFAULT_TEMPLATE = get_setting('PAGE_DEFAULT_TEMPLATE',
 PAGE_TEMPLATES = get_setting('PAGE_TEMPLATES',
     default_value=())
 
-# The callable that is used by the CMS
+
 def get_page_templates():
+    """The callable that is used by the CMS."""
     if callable(PAGE_TEMPLATES):
         return PAGE_TEMPLATES()
     else:
         return PAGE_TEMPLATES
 
 # Set ``PAGE_TAGGING`` to ``False`` if you do not wish to use the
-# ``django-tagging`` application.
+# ``django-taggit application.
 PAGE_TAGGING = getattr(settings, 'PAGE_TAGGING', False)
 if PAGE_TAGGING and "taggit" not in getattr(settings, 'INSTALLED_APPS', []):
     raise ImproperlyConfigured('django-taggit could not be found.\n'
@@ -78,6 +80,11 @@ PAGE_UNIQUE_SLUG_REQUIRED = getattr(settings, 'PAGE_UNIQUE_SLUG_REQUIRED',
 # pages revision information in the database
 PAGE_CONTENT_REVISION = getattr(settings, 'PAGE_CONTENT_REVISION', True)
 
+# Define the number of revisions too keep in the database. Set to None
+# if you want to keep everything
+PAGE_CONTENT_REVISION_DEPTH = getattr(settings,
+    'PAGE_CONTENT_REVISION_DEPTH', 10)
+
 # A list tuples that defines the languages that pages can be translated into.
 #
 # gettext_noop = lambda s: s
@@ -149,6 +156,10 @@ PAGE_HIDE_SITES = getattr(settings, 'PAGE_HIDE_SITES', False)
 PAGE_USE_LANGUAGE_PREFIX = getattr(settings, 'PAGE_USE_LANGUAGE_PREFIX',
                                                                         False)
 
+# Set this to True to raise an error 404 if the used URL path is
+# not strictly the same than the page.
+PAGE_USE_STRICT_URL = getattr(settings, 'PAGE_USE_STRICT_URL', False)
+
 # Assign a list of placeholders to PAGE_CONTENT_REVISION_EXCLUDE_LIST
 # to exclude them from the revision process.
 PAGE_CONTENT_REVISION_EXCLUDE_LIST = getattr(settings,
