@@ -1,5 +1,11 @@
+import os
+import sys
+coverage = None
+try:
+    from coverage import coverage
+except ImportError:
+    coverage = None
 
-import os, sys
 
 os.environ['DJANGO_SETTINGS_MODULE'] = 'pages.testproj.test_settings'
 current_dirname = os.path.dirname(__file__)
@@ -9,7 +15,6 @@ sys.path.insert(0, os.path.join(current_dirname, '../..'))
 from django.test.simple import run_tests as django_test_runner
 from django.db.models import get_app, get_apps
 
-from coverage import coverage
 import fnmatch
 
 # necessary for "python setup.py test"
@@ -22,14 +27,17 @@ patterns = (
     "pages.__init__",
     "pages.search_indexes",
     "pages.test_runner",
+    "pages.management.commands.*",
 )
 
+
 def match_pattern(filename):
     for pattern in patterns:
         if fnmatch.fnmatch(filename, pattern):
             return True
     return False
 
+
 def get_all_coverage_modules(app_module, exclude_patterns=[]):
     """Returns all possible modules to report coverage on, even if they
     aren't loaded.
@@ -57,24 +65,27 @@ def get_all_coverage_modules(app_module, exclude_patterns=[]):
 
     return mod_list
 
+
 def run_tests(test_labels=('pages',), verbosity=1, interactive=True,
         extra_tests=[]):
-            
-    cov = coverage()
-    cov.erase()
-    cov.use_cache(0)
-    cov.start()
+
+    if coverage:
+        cov = coverage()
+        cov.erase()
+        cov.use_cache(0)
+        cov.start()
+
     results = django_test_runner(test_labels, verbosity, interactive,
         extra_tests)
-    cov.stop()
 
-    app = get_app('pages')
-    modules = get_all_coverage_modules(app)
-    
-    cov.html_report(modules, directory='coverage')
+    if coverage:
+        cov.stop()
+        app = get_app('pages')
+        modules = get_all_coverage_modules(app)
+        cov.html_report(modules, directory='coverage')
 
     sys.exit(results)
-    
+
 
 if __name__ == '__main__':
-    run_tests()
\ No newline at end of file
+    run_tests()
