@@ -59,14 +59,14 @@ class FunctionnalTestCase(TestCase):
 
     def test_slug_collision(self):
         """Test a slug collision."""
-        setattr(settings, "PAGE_UNIQUE_SLUG_REQUIRED", True)
+        self.set_setting("PAGE_UNIQUE_SLUG_REQUIRED", True)
 
         c = self.get_admin_client()
 
         page_data = self.get_new_page_data()
         response = c.post('/admin/pages/page/add/', page_data)
         self.assertRedirects(response, '/admin/pages/page/')
-        setattr(settings, "PAGE_UNIQUE_SLUG_REQUIRED", False)
+        self.set_setting("PAGE_UNIQUE_SLUG_REQUIRED", False)
         response = c.post('/admin/pages/page/add/', page_data)
         self.assertEqual(response.status_code, 200)
 
@@ -237,8 +237,6 @@ class FunctionnalTestCase(TestCase):
         response = c.post('/admin/pages/page/%d/' % page.id, page_data)
         self.assertRedirects(response, '/admin/pages/page/')
 
-        #setattr(settings, "PAGE_DEFAULT_LANGUAGE", 'en-us')
-
         # test that the frontend view use the good parameters
         # I cannot find a way of setting the accept-language HTTP
         # header so I used django_language cookie instead
@@ -286,7 +284,7 @@ class FunctionnalTestCase(TestCase):
         response = c.get(page.get_url_path())
         self.assertContains(response, 'changed body 2', 1)
 
-        setattr(settings, "PAGE_CONTENT_REVISION", False)
+        self.set_setting("PAGE_CONTENT_REVISION", False)
 
         self.assertEqual(Content.objects.get_content(page, 'en-us', 'body'),
             'changed body 2')
@@ -311,7 +309,7 @@ class FunctionnalTestCase(TestCase):
         """
         Test diretory slugs
         """
-        setattr(settings, "PAGE_UNIQUE_SLUG_REQUIRED", False)
+        self.set_setting("PAGE_UNIQUE_SLUG_REQUIRED", False)
         c = self.get_admin_client()
         c.login(username='batiste', password='b')
 
@@ -535,8 +533,7 @@ class FunctionnalTestCase(TestCase):
         from pages.views import details
         from pages.utils import get_request_mock
         request = get_request_mock()
-        request.path = page1.get_url_path()
-        context = details(request, only_context=True)
+        context = details(request, path='/page1/', only_context=True)
         self.assertEqual(context['current_page'], page1)
 
 
@@ -741,6 +738,15 @@ class FunctionnalTestCase(TestCase):
         response = c.get('/pages/delegate')
         self.assertEqual(response.status_code, 200)
 
+        from pages.testproj.documents.models import Document
+        doc = Document(title='doc title 1', text='text', page=page)
+        doc.save()
+
+        response = c.get('/pages/delegate/doc-%d' % doc.id)
+        self.assertEqual(response.status_code, 200)
+
+        self.assertContains(response, "doc title 1")
+        reg.registry = []
 
     def test_untranslated(self):
         """Test the untranslated feature in the admin."""
@@ -786,3 +792,23 @@ class FunctionnalTestCase(TestCase):
         self.assertEqual(response.status_code, 200)
         response = c.get('/pages/other/')
         self.assertEqual(response.status_code, 200)
+
+    def test_page_sitemap(self):
+        """
+        Test the sitemap class
+        """
+        c = self.get_admin_client()
+        page1 = self.new_page(content={'slug':'english-slug'})
+        page1.save()
+        Content(page=page1, language='fr-ch', type='slug',
+            body='french-slug').save()
+
+        response = c.get('/sitemap.xml')
+        self.assertEqual(response.status_code, 200)
+        self.assertContains(response, 'english-slug')
+        self.assertNotContains(response, 'french-slug')
+
+        response = c.get('/sitemap2.xml')
+        self.assertEqual(response.status_code, 200)
+        self.assertContains(response, 'english-slug')
+        self.assertContains(response, 'french-slug')
