@@ -1,12 +1,14 @@
 """Default example views"""
 from django.http import Http404, HttpResponsePermanentRedirect
+from django.contrib.sitemaps import Sitemap
+from django.core.urlresolvers import resolve, Resolver404
+from django.utils import translation
 from pages import settings
 from pages.models import Page, PageAlias
-from pages.http import auto_render, get_language_from_request
-from pages.http import get_slug_and_relative_path
+from pages.http import auto_render, get_language_from_request, remove_slug
 from pages.urlconf_registry import get_urlconf
-from django.core.urlresolvers import resolve
-from django.utils import translation
+
+LANGUAGE_KEYS = [key for (key, value) in settings.PAGE_LANGUAGES]
 
 
 class Details(object):
@@ -21,14 +23,21 @@ class Details(object):
             **kwargs):
 
         current_page = False
-        lang = self.choose_language(lang, request)
 
-        # if the path is not defined, we assume that the user
-        # is using the view in a non usual way and fallback onto the
-        # the full request path.
         if path is None:
-            path = request.path
+            raise ValueError(
+                "pages.views.Details class view requires the path argument. "
+                "Check your urls.py file.")
 
+        # for the ones that might have forgotten to pass the language
+        # the language is now removed from the page path
+        if settings.PAGE_USE_LANGUAGE_PREFIX and lang is None:
+            maybe_lang = path.split("/")[0]
+            if maybe_lang in LANGUAGE_KEYS:
+                lang = maybe_lang
+                path = path[(len(lang) + 1):]
+
+        lang = self.choose_language(lang, request)
         pages_navigation = self.get_navigation(request, path, lang)
 
         context = {
@@ -65,9 +74,10 @@ class Details(object):
 
         self.extra_context(request, context)
 
-        answer = self.delegate(request, context, delegation)
-        if answer:
-            return answer
+        if delegation and current_page.delegate_to:
+            answer = self.delegate(request, context, delegation, **kwargs)
+            if answer:
+                return answer
 
         return template_name, context
 
@@ -75,12 +85,24 @@ class Details(object):
         """Return the appropriate page according to the path."""
         path = context['path']
         lang = context['lang']
-        pages_navigation = context['pages_navigation']
-        if path:
-            return Page.objects.from_path(path, lang,
+        page = Page.objects.from_path(path, lang,
+            exclude_drafts=(not is_staff))
+        if page:
+            return page
+        # if the complete path didn't worked out properly we gonna
+        # try to see if it might be a delegation page.
+        # To do that we remove the right part of the url and try again
+        # to find a page that match
+        path = remove_slug(path)
+        while path is not None:
+            page = Page.objects.from_path(path, lang,
                 exclude_drafts=(not is_staff))
-        elif pages_navigation:
-            return Page.objects.root()[0]
+            # find a match. Is the page delegating?
+            if page:
+                if page.delegate_to:
+                    return page
+            path = remove_slug(path)
+        return page
 
     def resolve_alias(self, request, path, lang):
         alias = PageAlias.objects.from_path(request, path, lang)
@@ -139,13 +161,24 @@ class Details(object):
         # if there is a delegation to another view,
         # call this view instead.
         current_page = context['current_page']
-        if delegation and current_page.delegate_to:
-            urlconf = get_urlconf(current_page.delegate_to)
-            result = resolve('/', urlconf)
-            if result:
-                view, args, kwargs = result
-                kwargs.update(context)
-                return view(request, *args, **kwargs)
+        path = context['path']
+        delegate_path = path.replace(current_page.get_complete_slug(), "")
+        # it seems that the urlconf path have to start with a slash
+        if len(delegate_path) == 0:
+            delegate_path = "/"
+        if delegate_path.startswith("//"):
+            delegate_path = delegate_path[1:]
+        urlconf = get_urlconf(current_page.delegate_to)
+        try:
+            result = resolve(delegate_path, urlconf)
+        except Resolver404:
+            raise Http404
+        if result:
+            view, args, kwargs = result
+            kwargs.update(context)
+            # for now the view is called as is. Usage of
+            # the auto_render decorator could simplify a few things.
+            return view(request, *args, **kwargs)
 
 
 # This view instance use the auto_render decorator. It means
@@ -156,3 +189,43 @@ class Details(object):
 #   >>> from pages.views import details
 #   >>> context = details(request, only_context=True)
 details = auto_render(Details())
+
+
+class PageSitemap(Sitemap):
+    """This site map implementation expose the pages
+    in the default language only."""
+    changefreq = "weekly"
+    priority = 0.5
+
+    def items(self):
+        return Page.objects.published()
+
+    def lastmod(self, obj):
+        return obj.last_modification_date
+
+
+class PageItemProxy(object):
+
+    def __init__(self, page, lang):
+        self.page = page
+        self.lang = lang
+
+    def get_absolute_url(self):
+        return self.page.get_absolute_url(language=self.lang)
+
+
+class MultiLanguagePageSitemap(Sitemap):
+    """This site map implementation expose the pages
+    in all the languages."""
+    changefreq = "weekly"
+    priority = 0.5
+
+    def items(self):
+        item_list = []
+        for page in Page.objects.published():
+            for lang in page.get_languages():
+                item_list.append(PageItemProxy(page, lang))
+        return item_list
+
+    def lastmod(self, obj):
+        return obj.page.last_modification_date
