@@ -26,6 +26,7 @@ register_widget(AdminTextareaWidget)
 
 if "filebrowser" in getattr(settings, 'INSTALLED_APPS', []):
     from filebrowser.fields import FileBrowseWidget
+
     class FileBrowseInput(FileBrowseWidget):
         """FileBrowseInput widget."""
 
@@ -89,6 +90,7 @@ if PAGE_TINYMCE:
             super(TinyMCE, self).__init__(language, attrs, mce_attrs)
     register_widget(TinyMCE)
 
+
 class CKEditor(Textarea):
     """CKEditor widget."""
 
@@ -117,6 +119,7 @@ class CKEditor(Textarea):
 
 register_widget(CKEditor)
 
+
 class WYMEditor(Textarea):
     """WYMEditor widget."""
 
@@ -149,7 +152,7 @@ class WYMEditor(Textarea):
             'PAGES_MEDIA_URL': PAGES_MEDIA_URL,
         }
         context['page_link_wymeditor'] = 1
-        context['page_list'] = Page.objects.all().order_by('tree_id','lft')
+        context['page_list'] = Page.objects.all().order_by('tree_id', 'lft')
 
         context['filebrowser'] = 0
         if "filebrowser" in getattr(settings, 'INSTALLED_APPS', []):
@@ -160,6 +163,7 @@ class WYMEditor(Textarea):
 
 register_widget(WYMEditor)
 
+
 class markItUpMarkdown(Textarea):
     """markItUpMarkdown widget."""
 
@@ -185,6 +189,33 @@ class markItUpMarkdown(Textarea):
             'pages/widgets/markitupmarkdown.html', context))
 register_widget(markItUpMarkdown)
 
+
+class markItUpRest(Textarea):
+    """markItUpRest widget."""
+
+    class Media:
+        js = [join(PAGES_MEDIA_URL, path) for path in (
+            'javascript/jquery.js',
+            'markitup/jquery.markitup.js',
+            'markitup/sets/rst/set.js',
+        )]
+        css = {
+            'all': [join(PAGES_MEDIA_URL, path) for path in (
+                'markitup/skins/simple/style.css',
+                'markitup/sets/rst/style.css',
+            )]
+        }
+
+    def render(self, name, value, attrs=None, **kwargs):
+        rendered = super(markItUpRest, self).render(name, value, attrs)
+        context = {
+            'name': name,
+        }
+        return rendered + mark_safe(render_to_string(
+            'pages/widgets/markituprest.html', context))
+register_widget(markItUpRest)
+
+
 class markItUpHTML(Textarea):
     """markItUpHTML widget."""
 
@@ -236,6 +267,7 @@ class EditArea(Textarea):
             'pages/widgets/editarea.html', context))
 register_widget(EditArea)
 
+
 class ImageInput(FileInput):
 
     def __init__(self, page=None, language=None, attrs=None, **kwargs):
@@ -280,7 +312,7 @@ class VideoWidget(MultiWidget):
     def value_from_datadict(self, data, files, name):
         value = [u'', u'', u'']
         for da in filter(lambda x: x.startswith(name), data):
-            index = int(da[len(name)+1:])
+            index = int(da[len(name) + 1:])
             value[index] = data[da]
         if value[0] == value[1] == value[2] == u'':
             return None
@@ -320,10 +352,10 @@ class LanguageChoiceWidget(TextInput):
     def render(self, name, value, attrs=None, **kwargs):
         context = {
             'name': name,
-            'value':value,
-            'page':self.page,
+            'value': value,
+            'page': self.page,
             'language': value,
-            'page_languages':PAGE_LANGUAGES
+            'page_languages': PAGE_LANGUAGES
         }
         return mark_safe(render_to_string(
             'pages/widgets/languages.html', context))
